/**
 * DistributionSelectionPanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import uk.co.petertribble.jingle.JingleTextPane;

public class DistributionSelectionPanel extends JPanel
					implements ActionListener {

    private static final int ACTION_REMOVE = 0;
    private static final int ACTION_ADD = 1;
    private static final int ACTION_PART_REMOVE = 2;

    private JingleTextPane tp_info;
    private JButton selectButton;

    private PackageProfile profile;
    private PkgList plist;
    private ClusterToc ctoc;

    private int chosen_action = ACTION_REMOVE;
    private Object chosen_object;

    public DistributionSelectionPanel(PackageProfile profile, PkgList plist,
				ClusterToc ctoc) {
	this.profile = profile;
	this.plist = plist;
	this.ctoc = ctoc;

	setLayout(new BorderLayout());

	tp_info = new JingleTextPane();
	add(new JScrollPane(tp_info));

	selectButton = new JButton("");
	selectButton.setEnabled(false);
	selectButton.addActionListener(this);
	add(selectButton, BorderLayout.SOUTH);

	setMinimumSize(new Dimension(200, 100));
    }

    /*
     * Packages and Clusters declare their dependencies as Sets of Strings,
     * so we need to convert those into Sets of Packages.
     */
    private Set <SolarisPackage> getDependencies(SolarisPackage pkg) {
	return getDependencies(pkg.getDependencySet());
    }
    private Set <SolarisPackage> getDependencies(PackageCluster pc) {
	return getDependencies(pc.getDependencySet());
    }
    private Set <SolarisPackage> getDependencies(Set <String> deps) {
	Set <SolarisPackage> pkgdeps = new HashSet <SolarisPackage> ();
	for (String s : deps) {
	    SolarisPackage pkg = plist.getPackage(s);
	    /*
	     * pkg shouldn't be null, but a common cause is
	     * depending on architecture-specific packages that
	     * don't exactly match by name.
	     */
	    if (pkg != null) {
		pkgdeps.add(pkg);
	    }
	}
	return pkgdeps;
    }

    /*
     * For much the same reason as above, we need to work out here
     * which dependencies are unselected.
     */
    private Set <SolarisPackage> getUnselectedDependencies(
					Set <SolarisPackage> deps) {
	Set <SolarisPackage> pkgdeps = new HashSet <SolarisPackage> ();
	for (SolarisPackage pkg : deps) {
	    if (!pkg.getSelected()) {
		pkgdeps.add(pkg);
	    }
	}
	return pkgdeps;
    }

    /*
     * Display a table of packages, one package per line. If the
     * packages is contained in a cluster, show that too.
     */
    private String pkgTableList(Set <SolarisPackage> pkgset) {
	StringBuilder sb = new StringBuilder();
	sb.append("<hr><table>");
	for (SolarisPackage pkg : pkgset) {
	    sb.append("<tr><td>").append(pkg.getName());
	    boolean first = true;
	    for (PackageCluster pc : ctoc.containingClusters(pkg)) {
		if (first) {
		    sb.append(" contained in cluster ");
		    first = false;
		} else {
		    sb.append(", ");
		}
		sb.append(pc.getClusterName());
	    }
	    sb.append("</td></tr>\n");
	}
	sb.append("</table><hr>");
	return sb.toString();
    }

    // FIXME localize
    // FIXME handle partially selected
    // FIXME handle clusters with mandatory packages
    public void showSelection(PackageCluster pc) {
	StringBuilder sb = new StringBuilder();
	sb.append("Cluster ").append(pc.getClusterName());
	if (pc.mustBeSelected()) {
	    sb.append(" is required and cannot be unselected");
	    disableButton();
	} else {
	    if (pc.getSelected()) {
		sb.append(" is currently selected");
		Set <SolarisPackage> pkgset = pc.getSelectedDependantSet();
		int i = pkgset.size();
		if (i > 0) {
		    sb.append(" and has ").append(i);
		    sb.append(" selected dependants");
		    sb.append(pkgTableList(pkgset));
		    disableButton();
		} else {
		    /*
		     * If the cluster contains some mandatory packages, only
		     * allow the user to remove the other packages.
		     */
		    if (pc.mustBePartiallySelected()) {
			enableButton(pc, ACTION_PART_REMOVE);
		    } else {
			enableButton(pc, ACTION_REMOVE);
		    }
		}
	    } else {
		sb.append(" is currently unselected");
		Set <SolarisPackage> pkgset =
		    getUnselectedDependencies(getDependencies(pc));
		int i = pkgset.size();
		if (i > 0) {
		    sb.append(" and has ").append(i);
		    sb.append(" unselected dependencies");
		    sb.append(pkgTableList(pkgset));
		}
		enableButton(pc, ACTION_ADD);
	    }
	}
	tp_info.setText(sb.toString());
    }

    // FIXME localize
    public void showSelection(SolarisPackage pkg) {
	StringBuilder sb = new StringBuilder();
	sb.append("Package ").append(pkg.getName());
	if (pkg.mustBeSelected()) {
	    sb.append(" is required and cannot be unselected");
	    disableButton();
	} else {
	    if (pkg.getSelected()) {
		sb.append(" is currently selected");
		Set <SolarisPackage> pkgset = pkg.getSelectedDependantSet();
		int i = pkgset.size();
		if (i > 0) {
		    sb.append(" and has ").append(i);
		    sb.append(" selected dependants");
		    sb.append(pkgTableList(pkgset));
		    disableButton();
		} else {
		    enableButton(pkg, ACTION_REMOVE);
		}
	    } else {
		sb.append(" is currently unselected");
		Set <SolarisPackage> pkgset =
		    getUnselectedDependencies(getDependencies(pkg));
		int i = pkgset.size();
		if (i > 0) {
		    sb.append(" and has ").append(i);
		    sb.append(" unselected dependencies");
		    sb.append(pkgTableList(pkgset));
		}
		enableButton(pkg, ACTION_ADD);
	    }
	}
	tp_info.setText(sb.toString());
    }

    /*
     * Here we add items recursively to the profile in order to resolve
     * dependencies
     */
    private void addRecursively(PackageCluster pc) {
	pc.setSelected(true);
	profile.addCluster(pc);
	Set <SolarisPackage> pkgset =
	    getUnselectedDependencies(getDependencies(pc));
	for (SolarisPackage pkg : pkgset) {
	    addRecursively(pkg);
	}
    }
    private void addRecursively(SolarisPackage pkg) {
	pkg.setSelected(true);
	profile.addPackage(pkg);
	Set <SolarisPackage> pkgset =
	    getUnselectedDependencies(getDependencies(pkg));
	for (SolarisPackage apkg : pkgset) {
	    addRecursively(apkg);
	}
    }
    /*
     * Remove the packages from a cluster that aren't mandatory.
     */
    private void removePartially(PackageCluster pc) {
	pc.setSelected(false);
	for (SolarisPackage pkg : pc.getPackages()) {
	    if (!pkg.mustBeSelected()) {
		pkg.setSelected(false);
		profile.removePackage(pkg);
	    }
	}
    }

    /*
     * Here we manipulate the state and effect of the selection button
     */
    private void disableButton() {
	selectButton.setText("");
	selectButton.setEnabled(false);
    }

    private void enableButton(SolarisPackage pkg, int act) {
	chosen_action = act;
	chosen_object = pkg;
	if (act == ACTION_ADD) {
	    enableButton("Add package " + pkg.getName());
	    // FIXME ...and its dependencies, if appropriate
	} else {
	    enableButton("Remove package " + pkg.getName());
	}
    }

    private void enableButton(PackageCluster pc, int act) {
	chosen_action = act;
	chosen_object = pc;
	if (act == ACTION_ADD) {
	    enableButton("Add cluster " + pc.getClusterName());
	    // FIXME ...and its dependencies, if appropriate
	} else if (act == ACTION_PART_REMOVE) {
	    enableButton("Remove removable packages from " +
			pc.getClusterName());
	} else {
	    enableButton("Remove cluster " + pc.getClusterName());
	}
    }

    private void enableButton(String s) {
	selectButton.setText(s);
	selectButton.setEnabled(true);
    }

    public void actionPerformed(ActionEvent e) {
	if (chosen_object instanceof SolarisPackage) {
	    SolarisPackage pkg = (SolarisPackage) chosen_object;
	    if (chosen_action == ACTION_ADD) {
		addRecursively(pkg);
	    } else {
		pkg.setSelected(false);
		profile.removePackage(pkg);
	    }
	    showSelection(pkg);
	} else {
	    PackageCluster pc = (PackageCluster) chosen_object;
	    if (chosen_action == ACTION_ADD) {
		addRecursively(pc);
	    } else if (chosen_action == ACTION_PART_REMOVE) {
		removePartially(pc);
	    } else {
		pc.setSelected(false);
		profile.removeCluster(pc);
	    }
	    showSelection(pc);
	}
    }
}
