/**
 * InstalledClusterPanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import javax.swing.event.*;

import org.jdesktop.swingworker.SwingWorker;

public class InstalledClusterPanel extends JPanel {

    // Like ClusterPanel, but for the currently installed system

    private PkgList plist;
    private ClusterToc ctoc;

    private PackageInformationPanel pip;

    private SolarisPackage currentPackage = null;

    private ContentsParser cp;

    public InstalledClusterPanel(PkgList plist, ClusterToc ctoc) {
	this.plist = plist;
	this.ctoc = ctoc;

	setLayout(new BorderLayout());

	// Which cluster do we have installed?
	InstalledCluster iclust = new InstalledCluster();
	String instcluster = iclust.getClusterName();

	// tree view of clusters
	String toptext = instcluster;
	if (instcluster == null) {
	    toptext = "Solaris";
	}
	DefaultMutableTreeNode topnode = new DefaultMutableTreeNode(toptext);
	final JTree tree = new JTree(topnode);
	if (instcluster == null) {
	    ClusterTree ct = new ClusterTree(topnode, ctoc);
	} else {
	    ClusterTree ct = new ClusterTree(topnode, ctoc, instcluster);
	}
	tree.expandRow(0);
	// set a renderer to do tooltips
	tree.setCellRenderer(new ClusterTreeCellRenderer(
				ClusterTreeCellRenderer.ICON_INSTALLED));
	ToolTipManager.sharedInstance().registerComponent(tree);
	// Listen for when the selection changes.
	tree.addTreeSelectionListener(new TreeSelectionListener() {
	    public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		    tree.getLastSelectedPathComponent();

		if (node != null) {
		    if (node.isLeaf()) {
			setPackageInfo((SolarisPackage) node.getUserObject());
		    } else {
			Object o = node.getUserObject();
			if (o instanceof String) {
			    setClusterInfo(o.toString());
			} else {
			    setClusterInfo((PackageCluster) o);
			}
		    }
		}
	    }
	});
	// end of tree listener

	// tabbed info pane on right
	pip = new PackageInformationPanel();

	// split pane to hold the lot
	JSplitPane psplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
		new JScrollPane(tree), pip);
	psplit.setOneTouchExpandable(true);
	psplit.setDividerLocation(180);
	add(psplit);
    }

    private void setClusterInfo(String s) {
	currentPackage = null;
	MetaCluster mc = ctoc.getMetaCluster(s);
	if (mc != null) {
	    StringBuilder sb = new StringBuilder();
	    sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	    sb.append("<th>MetaCluster Name</th></tr>\n");
	    sb.append("<tr><td>").append(mc.getName()).append("</td></tr>\n");
	    sb.append("</table>\n");
	    pip.setInfoText(sb.toString());
	    pip.setClusterText("");
	} else {
	    pip.setInfoText(s);
	}
    }

    private void setClusterInfo(PackageCluster pc) {
	currentPackage = null;
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th>Cluster Name</th></tr>\n");
	sb.append("<tr><td>").append(pc.getName()).append("</td></tr>\n");
	sb.append("<tr bgcolor=\"#eeeeee\">");
	sb.append("<th>Installation Status</th></tr>\n");
	sb.append("<tr><td>").append(pc.installedStatusText());
	sb.append("</td></tr>\n");
	sb.append("</table>\n");
	sb.append(PkgUtils.detailTable(pc, cp));
	pip.setInfoText(sb.toString(), PkgUtils.dependencyTable(pc, plist));
	setMetaClusterMembership(pc);
    }

    /**
     * Display which clusters and metaclusters a given package is in.
     */
    private void setClusterMembership(SolarisPackage pkg) {
	pip.setClusterText(PkgUtils.clusterMembership(pkg, ctoc));
    }

    /**
     * Display which metaclusters a given cluster is in.
     */
    private void setMetaClusterMembership(PackageCluster pc) {
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\">");
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append("This cluster is part of the following Metaclusters:");
	sb.append("</th></tr>\n");
	for (MetaCluster mcluster : ctoc.containingMetaClusters(pc)) {
	    sb.append("<tr><td>").append(mcluster).append("</td><td>");
	    sb.append(mcluster.getName()).append("</td></tr>\n");
	}
	sb.append("</table>\n");
	pip.setClusterText(sb.toString());
    }

    private void setPackageInfo(SolarisPackage pkg) {
	if (pkg.exists()) {
	    currentPackage = pkg;
	    pip.setInfoText(PkgUtils.infoTable(pkg)
				+ PkgUtils.detailTable(pkg, cp),
		    PkgUtils.dependencyTable(pkg, plist),
		    plist.revDeps(pkg.getName()));
	} else {
	    pip.setInfoText("Not installed");
	}
	setClusterMembership(pkg);
    }

    public void createRevDependencies() {
	(new RevDependencyWorker()).execute();
    }

    class RevDependencyWorker extends SwingWorker <String, Object> {
	@Override
	public String doInBackground() {
	    plist.createRevDependencies();
	    return "done";
	}

	@Override
	protected void done() {
	    try {
		pip.showRevDependencies();
		if (currentPackage != null) {
		    setPackageInfo(currentPackage);
		}
	    } catch (Exception ignore) {
	    }
	}
    }

    public void showDetailedView() {
	cp = ContentsParser.getInstance();
    }
}
