/**
 * ProfileBuilder - interactively construct a jumpstart profile
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import uk.co.petertribble.jingle.JingleInfoFrame;

public class ProfileBuilder implements ActionListener {

    private PackageProfile profile;
    private DistributionSoftwarePanel cpanel;

    private JMenuItem exitItem;
    private JMenuItem helpItem;
    private JMenuItem licenseItem;
    private JButton confirmButton;
    private JButton useLocalButton;
    private JTabbedPane jtp;

    private String useCluster;
    private InstalledCluster iclust;

    private static final int CHOOSE_TABID = 0;
    private static final int PACKAGE_TABID = 1;
    private static final int PROFILE_TABID = 2;

    public ProfileBuilder(String distdir) {
	PkgResources pkgres = PkgResources.getInstance();

	JFrame jf = new JFrame("ProfileBuilder");
	jf.addWindowListener(new winExit());

	jtp = new JTabbedPane();
	jf.getContentPane().add(jtp, BorderLayout.CENTER);

	JMenuBar jm = new JMenuBar();

	JMenu jme = new JMenu(pkgres.getString("FILE.TEXT"));
	jme.setMnemonic(KeyEvent.VK_F);
	exitItem = new JMenuItem(pkgres.getString("FILE.EXIT.TEXT"),
				KeyEvent.VK_X);
	exitItem.addActionListener(this);
	jme.add(exitItem);

	JMenu jmh = new JMenu(pkgres.getString("HELP.TEXT"));
	jmh.setMnemonic(KeyEvent.VK_H);
	helpItem = new JMenuItem(pkgres.getString("HELP.ABOUT.TEXT")
				+ " solview", KeyEvent.VK_A);
	helpItem.addActionListener(this);
	jmh.add(helpItem);
	licenseItem = new JMenuItem(pkgres.getString("HELP.LICENSE.TEXT"),
				KeyEvent.VK_L);
	licenseItem.addActionListener(this);
	jmh.add(licenseItem);

	jm.add(jme);
	jm.add(jmh);
	jf.setJMenuBar(jm);

	PkgList plist = new PkgList(distdir);
	ClusterToc ctoc = new ClusterToc(distdir, plist);

	profile = new PackageProfile(ctoc);

	JPanel mp = new JPanel(new BorderLayout());
	ProfileDisplayPanel pdp = new ProfileDisplayPanel(profile);

	/*
	 * The tabs are used for workflow, hence the need to be precise
	 * in the index.
	 */
	jtp.insertTab(pkgres.getString("JS.TAB.MC"), (Icon) null, mp,
		(String) null, CHOOSE_TABID);

	cpanel = new DistributionSoftwarePanel(ctoc, plist, true, profile);
	jtp.insertTab(pkgres.getString("JS.TAB.PKG"), (Icon) null, cpanel,
		(String) null, PACKAGE_TABID);

	jtp.insertTab(pkgres.getString("JS.TAB.OUT"), (Icon) null, pdp,
		(String) null, PROFILE_TABID);

	/*
	 * Initially the package selection and output screens are disabled,
	 * and remain so until the user selects a starting metacluster.
	 */
	jtp.setEnabledAt(PACKAGE_TABID, false);
	jtp.setEnabledAt(PROFILE_TABID, false);

	jf.setSize(720, 540);
	jf.setVisible(true);

	/*
	 * Now we start initialising.
	 */
	JLabel statuslabel = new JLabel(pkgres.getString("JS.MSG.INIT"));
	mp.add(statuslabel, BorderLayout.NORTH);

	// FIXME: really ought to use a SwingWorker here
	statuslabel.setText(pkgres.getString("JS.MSG.DEP"));
	statuslabel.setText(pkgres.getString("JS.MSG.SEL"));

	// which cluster is installed
	iclust = new InstalledCluster();

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.PAGE_AXIS));
	ButtonGroup bg = new ButtonGroup();
	for (String s : ctoc.getMetaClusters().keySet()) {
	    JRadioButton jb = new JRadioButton(s);
	    jb.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    confirmButton.setEnabled(true);
		    useCluster = ((JRadioButton) e.getSource()).getText();
		}
	    });
	    bg.add(jb);
	    buttonPanel.add(jb);
	}
	mp.add(new JScrollPane(buttonPanel));

	JPanel confirmPanel = new JPanel();
	confirmButton = new JButton(pkgres.getString("JS.MSG.CUSTOM"));
	confirmButton.addActionListener(this);
	confirmButton.setEnabled(false);
	confirmPanel.add(confirmButton);
	if (iclust.isDefined()) {
	    String sclust = pkgres.getString("JS.MSG.USELOCAL") + " ("
				+ iclust.getClusterName() + ")";
	    useLocalButton = new JButton(sclust);
	    useLocalButton.addActionListener(this);
	    useLocalButton.setEnabled(true);
	    confirmPanel.add(useLocalButton);
	}
	mp.add(confirmPanel, BorderLayout.SOUTH);
	mp.validate();

    }

    static class winExit extends WindowAdapter {
	public void windowClosing(WindowEvent we) {
	    System.exit(0);
	}
    }

    // FIXME: also unset all the radio buttons
    private void reset() {
	jtp.setEnabledAt(PACKAGE_TABID, false);
	jtp.setEnabledAt(PROFILE_TABID, false);
	jtp.setSelectedIndex(CHOOSE_TABID);
	jtp.setEnabledAt(CHOOSE_TABID, true);
	useCluster = null;
    }

    public void actionPerformed(ActionEvent e) {
	if (e.getSource() == confirmButton) {
	    cpanel.setMetaCluster(useCluster);
	    jtp.setEnabledAt(PACKAGE_TABID, true);
	    jtp.setEnabledAt(PROFILE_TABID, true);
	    jtp.setSelectedIndex(PACKAGE_TABID);
	    jtp.setEnabledAt(CHOOSE_TABID, false);
	}
	if (e.getSource() == useLocalButton) {
	    cpanel.useInstalledSystem(iclust);
	    jtp.setEnabledAt(PACKAGE_TABID, true);
	    jtp.setEnabledAt(PROFILE_TABID, true);
	    jtp.setSelectedIndex(PACKAGE_TABID);
	    jtp.setEnabledAt(CHOOSE_TABID, false);
	}
	if (e.getSource() == exitItem) {
	    System.exit(0);
	}
	if (e.getSource() == helpItem) {
	    new JingleInfoFrame(this.getClass().getClassLoader(),
				"help/profile.html", "text/html");
	}
	if (e.getSource() == licenseItem) {
	    new JingleInfoFrame(this.getClass().getClassLoader(),
				"help/CDDL.txt", "text/plain");
	}
    }

    public static void main(String args[]) {
	if (args.length != 1) {
	    System.out.println("Usage: ProfileBuilder product_dir");
	    System.exit(1);
	}
	File f = new File(args[0], ".clustertoc");
	if (f.exists()) {
	    new ProfileBuilder(args[0]);
	} else {
	    System.out.println("Usage: ProfileBuilder product_dir");
	    System.exit(1);
	}
    }
}
