/**
 * ProfileParser - import and parse a jumpstart profile
 * Can also validate it against a distribution directory
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import uk.co.petertribble.jumble.JumbleFile;
import java.io.*;
import java.util.*;

public class ProfileParser {

    private StringBuilder header;
    private StringBuilder trailer;

    private String mcluster;
    private Set <String> addClusters;
    private Set <String> deleteClusters;
    private Set <String> addPackages;
    private Set <String> deletePackages;

    private PkgList plist;
    private ClusterToc ctoc;

    private static final String pkgstr = "package";
    private static final String clstr = "cluster";

    // refers to syntax
    private boolean isvalid = true;
    // refers to profile content
    private boolean haserrors = false;
    private boolean haswarnings = false;
    // hold reports
    private StringBuilder errors;
    private StringBuilder warnings;

    public ProfileParser(String filename) {
	this(new File(filename), (String) null);
    }

    public ProfileParser(String filename, String distdir) {
	this(new File(filename), distdir);
    }

    public ProfileParser(File f) {
	this(f, (String) null);
    }

    public ProfileParser(File profile, String distdir) {
	header = new StringBuilder();
	trailer = new StringBuilder();
	errors = new StringBuilder();
	warnings = new StringBuilder();

	addClusters = new HashSet <String> ();
	addPackages = new HashSet <String> ();
	deleteClusters = new HashSet <String> ();
	deletePackages = new HashSet <String> ();

	String[] lines = JumbleFile.getStringContents(profile).split("\n");
	boolean inheader = true;
	for (int i = 0; i < lines.length; i++) {
	    String thisline = lines[i].trim();
	    if (thisline.startsWith(clstr)) {
		// cluster selection, so no longer in a header
		inheader = false;
		/*
		 * The algorithm here is that we split the line into words.
		 * If 2 words, then it ought to be defining the metacluster.
		 * If 3 words, then the 3rd word must be add or delete.
		 */
		String[] ds = thisline.split("\\s+");
		if (ds.length == 2) {
		    // define the metacluster
		    mcluster = ds[1];
		    // FIXME check that it looks like a metacluster
		} else if (ds.length == 3) {
		    // add or delete a cluster
		    if (ds[2].equals("add")) {
			addClusters.add(ds[1]);
		    } else if (ds[2].equals("delete")) {
			deleteClusters.add(ds[1]);
		    } else {
			isvalid = false;
			addError("Invalid cluster line:\n" + thisline);
		    }
		} else {
		    isvalid = false;
		    addError("Invalid cluster line:\n" + thisline);
		}
	    } else if (thisline.startsWith(pkgstr)) {
		// package selection, so no longer in a header
		inheader = false;
		/*
		 * The algorithm here is that we split the line into words.
		 * If 3 words, then the 3rd word must be add or delete.
		 */
		String[] ds = thisline.split("\\s+");
		if (ds.length == 3) {
		    // add or delete a package
		    if (ds[2].equals("add")) {
			addPackages.add(ds[1]);
		    } else if (ds[2].equals("delete")) {
			deletePackages.add(ds[1]);
		    } else {
			isvalid = false;
			addError("Invalid package line:\n" + thisline);
		    }
		} else {
		    isvalid = false;
		    addError("Invalid package line:\n" + thisline);
		}
	    } else {
		if (inheader) {
		    header.append(thisline).append("\n");
		} else {
		    trailer.append(thisline).append("\n");
		}
	    }
	}
	// FIXME check to see we have exactly one metacluster
	// FIXME or none in the case of upgrade profiles
	/*
	 * If we have been given a distribution directory then we can import
	 * that and validate the profile we just read against it.
	 */
	// FIXME validate distdir
	if (distdir != null) {
	    checkProfile(distdir);
	}
    }

    private void checkProfile(String distdir) {
	plist = new PkgList(distdir);
	ctoc = new ClusterToc(distdir, plist);
	if (!ctoc.exists()) {
	    addError("Invalid distribution directory");
	}
	// is the metacluster valid?
	Set <String> mcs = ctoc.getMetaClusters().keySet();
	if (!mcs.contains(mcluster)) {
	    addError("Invalid metacluster " + mcluster);
	}
	/*
	 * For each cluster added or deleted by the profile, we can
	 * check whether it was in the specified metacluster or not
	 * and do the same for packages
	 */
	// FIXME only if it exists
	MetaCluster mc = ctoc.getMetaCluster(mcluster);
	// Check for duplicates
	Set <String> dupClusters = new HashSet <String>(addClusters);
	dupClusters.retainAll(deleteClusters);
	Set <String> dupPackages = new HashSet <String>(addPackages);
	dupPackages.retainAll(deletePackages);
	for (String s : dupClusters) {
	    addWarning("Cannot both add and delete cluster " + s);
	}
	for (String s : dupPackages) {
	    addWarning("Cannot both add and delete package " + s);
	}

	// if deleted, check it's present
	for (String cname : deleteClusters) {
	    if (ctoc.clusterExists(cname)) {
		if (!mc.containsCluster(cname)) {
		    addWarning("Cluster " + cname + " deleted unnecessarily");
		}
	    } else {
		addWarning("Unrecognized cluster " + cname);
	    }
	}
	// if added, check it wasn't already present
	for (String cname : addClusters) {
	    if (ctoc.clusterExists(cname)) {
		if (mc.containsCluster(cname)) {
		    addWarning("Cluster " + cname + " added unnecessarily");
		}
	    } else {
		addWarning("Unrecognized cluster " + cname);
	    }
	}

	/*
	 * Checking packages is a little more complicated. We not only need
	 * to check them against the metacluster, but also against the list
	 * of added and deleted clusters.
	 */
	for (String pname : deletePackages) {
	    // check it exists
	    if (plist.getPackage(pname) == null) {
		addWarning("Unrecognized package " + pname);
	    } else {
		for (String cname : deleteClusters) {
		    PackageCluster pc = ctoc.getCluster(cname);
		    if ((pc != null) && pc.containsPackage(pname)) {
			addWarning("Package " + pname + " already deleted via "
							+ cname);
		    }
		}
		if (!mc.includesPackage(pname)) {
		    /*
		     * Removing this package is wrong unless we added a cluster
		     * that pulled it in.
		     */
		    boolean packwarn = true;
		    for (String cname : addClusters) {
			PackageCluster pc = ctoc.getCluster(cname);
			if ((pc != null) && pc.containsPackage(pname)) {
			    packwarn = false;
			}
		    }
		    if (packwarn) {
			addWarning("Package " + pname +
						" deleted unnecessarily");
		    }
		}
	    }
	}
	for (String pname : addPackages) {
	    // check it exists
	    if (plist.getPackage(pname) == null) {
		addWarning("Unrecognized package " + pname);
	    } else {
		for (String cname : addClusters) {
		    PackageCluster pc = ctoc.getCluster(cname);
		    if ((pc != null) && pc.containsPackage(pname)) {
			addWarning("Package " + pname + " already included via "
+ cname);
		    }
		}
		if (mc.includesPackage(pname)) {
		    /*
		     * Adding this package is wrong unless we deleted a cluster
		     * that took it away and we're putting it back.
		     */
		    boolean packwarn = true;
		    for (String cname : deleteClusters) {
			PackageCluster pc = ctoc.getCluster(cname);
			if ((pc != null) && pc.containsPackage(pname)) {
			    packwarn = false;
			}
		    }
		    if (packwarn) {
			addWarning("Package " + pname + " added unnecessarily");
		    }
		}
	    }
	}
    }

    private void addError(String s) {
	haserrors = true;
	errors.append(s).append("\n");
    }

    private void addWarning(String s) {
	haswarnings = true;
	warnings.append(s).append("\n");
    }

    public boolean isValid() {
	return isvalid;
    }

    public boolean hasErrors() {
	return haserrors;
    }

    public boolean hasWarnings() {
	return haswarnings;
    }

    public String getErrors() {
	return errors.toString();
    }

    public String getWarnings() {
	return warnings.toString();
    }

    public String printProfile() {
	StringBuilder sb = new StringBuilder();
	// all the header from the original profile
	sb.append(header.toString());
	// walk through the software selection
	sb.append(clstr).append("\t").append(mcluster).append("\n");
	for (String s : addClusters) {
	    sb.append(clstr).append("\t").append(s).append("\tadd\n");
	}
	for (String s : deleteClusters) {
	    sb.append(clstr).append("\t").append(s).append("\tdelete\n");
	}
	for (String s : addPackages) {
	    sb.append(pkgstr).append("\t").append(s).append("\tadd\n");
	}
	for (String s : deletePackages) {
	    sb.append(pkgstr).append("\t").append(s).append("\tdelete\n");
	}
	// and the rest
	sb.append(trailer.toString());
	return sb.toString();
    }

    public static void main(String args[]) {
	if (args.length == 1) {
	    File f = new File(args[0]);
	    if (f.exists()) {
		ProfileParser pp = new ProfileParser(f);
		if (pp.isValid()) {
		    System.out.print(pp.printProfile());
		} else {
		    System.out.println("ERROR: invalid profile");
		    System.out.println(pp.getErrors());
		}
	    } else {
		System.out.println("ERROR: Unable to find input profile");
	    }
	} else if (args.length == 2) {
	    File f = new File(args[0]);
	    File ff = new File(args[1]);
	    if (f.exists() && ff.exists() && ff.isDirectory()) {
		ProfileParser pp = new ProfileParser(f, args[1]);
		if (pp.isValid()) {
		    System.out.print(pp.printProfile());
		    if (pp.hasWarnings()) {
			System.out.println("++++++++++++WARNINGS++++++++++++");
			System.out.println(pp.getWarnings());
			System.out.println("++++++++++END WARNINGS++++++++++");
		    }
		} else {
		    System.out.println("ERROR: invalid profile");
		    System.out.println(pp.getErrors());
		}
	    } else {
		if (!f.exists()) {
		    System.out.println("ERROR: Unable to find input profile");
		}
		if (!ff.exists()) {
		    System.out.println("ERROR: Cannot find Product directory");
		} else {
		    if (!ff.isDirectory()) {
			System.out.println(
				"ERROR: Product directory isn't a directory");
		    }
		}
	    }
	} else {
	    System.out.println("Usage: parseprofile profile [product_dir]");
	}
    }

}
