/**
 * SVMconfig - represent the configuration of SVM
 * @author Peter Tribble
 * @version 1.0
 *
 */

/*
 * I'm not sure how to handle metasets. presumably there's a parent class
 * of this one that holds the lists of metasets, or should that be here?
 */

package uk.co.petertribble.solview.explorer;

import java.util.*;
import uk.co.petertribble.solview.InfoCommand;

public class SVMconfig {

    private boolean configured = true;
    private MetaSet metaset;
    private Set <MetaDB> metadbs;
    private Set <MetaHS> metahotspares;
    private Set <MetaDevice> metadevices;
    private Map <String, MetaDevice> metamap;

    public SVMconfig() {
	this((MetaSet) null);
    }

    public SVMconfig(MetaSet metaset) {
	this.metaset = metaset;
	metadbs = new HashSet <MetaDB> ();
	metadevices = new HashSet <MetaDevice> ();
	metahotspares = new HashSet <MetaHS> ();
	metamap = new HashMap <String, MetaDevice> ();
	parse();
    }

    public MetaDevice getDevice(String name) {
	return metamap.get(name);
    }

    private void parse() {
	parse_metadb();
	parse_metastat();
    }

    private void parse_metadb() {
	InfoCommand ic = new InfoCommand("MD", "/usr/sbin/metadb");
	if (ic.exists()) {
	    String s = ic.getOutput();
	} else {
	    configured = false;
	}
    }

    /*
     * Parsing metastat output: we take two passes. In the first pass, we
     * simply enumerate the list of devices. In the second pass, we can then
     * assign prperties to build up relationships.
     */
    private void parse_metastat() {
	InfoCommand ic = new InfoCommand("MD", "/usr/sbin/metastat", "-p");
	if (ic.exists()) {
	    String[] lines = ic.getOutput().split("\n");
	    for (int i = 0; i < lines.length; i++) {
		String[] ds = lines[i].split("\\s+");
		/*
		 * Conventionally, metadevices start with d; the only other
		 * possible entries are hot spares starting with hsp.
		 */
		if (ds[0].startsWith("d")) {
		    // metadevice
		    MetaDevice md = parse_metadevice(ds);
		    metadevices.add(md);
		    metamap.put(ds[0], md);
		} else if (ds[0].startsWith("hsp")) {
		    // Hot spare pool
		    metahotspares.add(parse_hotspare(ds));
		} else {
		    System.err.println("Unhandled metastat output\n"
				+ lines[i]);
		}
	    }
	    for (int i = 0; i < lines.length; i++) {
		String[] ds = lines[i].split("\\s+");
		if (ds[0].startsWith("d")) {
		    relate_metadevices(ds);
		}
	    }
	} else {
	    configured = false;
	}
    }

    private MetaDevice parse_metadevice(String[] ds) {
	int mtype = 0;
	if (ds[1].equals("-m")) {
	    mtype = MetaDevice.MIRROR;
	} else if (ds[1].equals("-r")) {
	    mtype = MetaDevice.RAID5;
	} else if (ds[1].equals("-p")) {
	    mtype = MetaDevice.SOFTPART;
	} else if (ds[1].equals("1")) {
	    mtype = MetaDevice.STRIPE;
	} else {
	    mtype = MetaDevice.CONCAT;
	}
	return new MetaDevice(ds[0], mtype);
    }

    /*
     * Walk through the metadevices to define relationships
     */
    private void relate_metadevices(String[] ds) {
	MetaDevice md = metamap.get(ds[0]);
	if (md == null) {
	    return;
	}
	if (md.getType() == MetaDevice.MIRROR) {
	    /*
	     * format: d0 -m d1 [d2 [d3]] n
	     *
	     * For each submirror, add it as a child of the primary device
	     * and add the primary as the parent of each submirror
	     */
	    for (int i = 2; i < ds.length; i++) {
		MetaDevice md2 = metamap.get(ds[i]);
		if (md2 != null) {
		    md.addChild(md2);
		    md2.setParent(md);
		}
	    }
	}
    }

    private MetaHS parse_hotspare(String[] ds) {
	return new MetaHS();
    }

}
